@@ -29,16 +29,38 @@ use     strict;
 package ODF::lpOD::Table;
 use base 'ODF::lpOD::Element';
 our $VERSION    = 0.1;
-use constant PACKAGE_DATE => '2010-06-19T19:23:38';
+use constant PACKAGE_DATE => '2010-06-23T11:30:36';
 use ODF::lpOD::Common;
 #=============================================================================
 
 our $ROW_FILTER         = 'table:table-row';
 our $COLUMN_FILTER      = 'table:table-column';
 
-#-----------------------------------------------------------------------------
+#--- common utilities --------------------------------------------------------
 
-sub	translate_coordinates   # taken from OpenOffice::OODoc (Genicorp)
+sub     alpha_to_num
+        {
+        my $arg = shift         or return 0;
+        my $alpha = uc $arg;
+        unless ($alpha =~ /^[A-Z]*$/)
+                {
+                return $arg if $alpha =~ /^[0-9]*$/;
+                alert "Wrong value $arg";
+                return undef;
+                }
+
+        my @asplit = split('', $alpha);
+        my $num = 0;
+        foreach my $p (@asplit)
+                {
+		$num *= 26;
+		$num += ((ord($p) - ord('A')) + 1);
+                }
+        $num--;
+        return $num;
+        }
+
+sub	translate_coordinates   # adapted from OpenOffice::OODoc (Genicorp)
 	{
 	my $arg	= shift; return ($arg, @_) unless defined $arg;
 	my $coord = uc $arg;
@@ -50,19 +72,71 @@ sub	translate_coordinates   # taken from OpenOffice::OODoc (Genicorp)
 	my $r	= $2;
 	return ($arg, @_) unless ($c && $r);
 
-	my $rownum	= $r - 1;
-	my @csplit	= split '', $c;
-	my $colnum	= 0;
-	foreach my $p (@csplit)
-		{
-		$colnum *= 26;
-		$colnum	+= ((ord($p) - ord('A')) + 1);
-		}
-	$colnum--;
-
+	my $rownum = $r - 1;
+	my $colnum = alpha_to_num($c);
 	return ($rownum, $colnum, @_);
 	}
 
+sub     translate_range
+        {
+        my $arg = shift; return ($arg, @_) unless
+                                (defined $arg && $arg =~ /:/);
+        my $range = uc $arg;
+        $range =~ s/\s*//g;
+        my ($start, $end) = split(':', $range);
+        my @r = ();
+        for ($start, $end)
+                {
+                my $p = uc $_;
+                given ($p)
+                        {
+                        when (undef)
+                                {
+                                push @r, 0;
+                                }
+                        when (/^[A-Z]*$/)
+                                {
+                                push @r, alpha_to_num($p);
+                                }
+                        when (/^[0-9]*$/)
+                                {
+                                push @r, ($p - 1);
+                                }
+                        default
+                                {
+                                alert "Wrong range end $p";
+                                return undef;
+                                }
+                        }
+                }
+        return @r;
+        }
+
+sub     split_rep
+        {
+        my      ($elt1, $pos, $limit) = @_;
+        
+        my $reps = $elt1->get_repeated;
+        if ($reps > 1 && defined $limit && ($pos + $reps) > $limit)
+                {
+                my $beyond = ($pos + $reps) - $limit;
+                $reps -= $beyond;
+                my $elt2 = $elt1->next;
+                unless ($elt2)
+                        {
+                        $elt2 = $elt1->copy;
+                        $elt2->set_repeated($beyond + 1);
+                        $elt2->paste_after($elt1);
+                        }
+                else
+                        {
+                        $elt2->set_repeated($beyond + $elt2->get_repeated);
+                        }
+                }
+        $pos += $elt1->repeat($reps);
+        return $pos;
+        }
+
 #--- constructor -------------------------------------------------------------
 
 sub     create
@@ -110,6 +184,22 @@ sub     create
         return $t;
         }
 
+#--- internal method for row & cell repetiion limit --------------------------
+
+sub     set_working_area
+        {
+        my $self        = shift;
+        my ($h, $w)     = @_;
+        $self->set_attribute('#lpod:h' => $h);
+        $self->set_attribute('#lpod:w' => $w);
+        }
+
+sub     clean
+        {
+        my $self        = shift;
+        $_->clean() for $self->children($ROW_FILTER);
+        }
+
 #-----------------------------------------------------------------------------
 
 sub     get_row
@@ -117,6 +207,7 @@ sub     get_row
         my $self        = shift;
         my $position    = shift || 0;
         my $height      = $self->get_height;
+        my $max_h       = $self->att('#lpod:h');
 
         if ($position < 0)
                 {
@@ -131,12 +222,36 @@ sub     get_row
         my $row = $self->first_child($ROW_FILTER)
                 or return undef;
         my $p = 0;
+        my $next_elt;
         do      {
-                my $next_elt = $row->next_sibling($ROW_FILTER);
-                $p += $row->repeat(); $p++;
-                $row = $next_elt;
+                $next_elt = $row->next;          
+                $p = ODF::lpOD::Table::split_rep($row, $p, $max_h);
+                $p++; $row = $next_elt;
                 } until $p >= $position;
-        return $self->child($position, $ROW_FILTER);
+        $row = $self->child($position, $ROW_FILTER);     
+        ODF::lpOD::Table::split_rep($row, $p, $max_h);
+        return $row;
+        }
+
+sub     get_row_list
+        {
+        my $self        = shift;
+        my $arg         = shift;
+        my ($start, $end);
+        if ($arg)
+                {
+                ($start, $end) = translate_range($arg, shift);
+                }
+        $start //= 0; $end //= -1;
+        my @list = ();
+        my $elt = $self->get_row($start);
+        my $last_elt = $self->get_row($end);
+        while ($elt && ! $elt->after($last_elt))
+                {
+                push @list, $elt;
+                $elt = $elt->next;
+                }
+        return @list;
         }
 
 sub     get_column
@@ -144,6 +259,8 @@ sub     get_column
         my $self        = shift;
         my $position    = shift || 0;
         my $width       = $self->get_column_count;
+        my $max_w       = $self->get_attribute('#lpod:w');
+
         if ($position < 0)
                 {
                 $position += $width;
@@ -157,22 +274,100 @@ sub     get_column
         my $col = $self->first_child($COLUMN_FILTER)
                 or return undef;
         my $p = 0;
+        my $next_elt;
         do      {
-                my $next_elt = $col->next_sibling($COLUMN_FILTER);
-                $p += $col->repeat(); $p++;
-                $col = $next_elt;
+                $next_elt = $col->next_sibling($COLUMN_FILTER);
+                $p = ODF::lpOD::Table::split_rep($col, $p, $max_w);
+                $p++; $col = $next_elt;
                 } until $p >= $position;
-        return $self->child($position, $COLUMN_FILTER);
+        $col = $self->child($position, $COLUMN_FILTER);
+        ODF::lpOD::Table::split_rep($col, $p, $max_w);
+        return $col;
+        }
+
+sub     get_column_list
+        {
+        my $self        = shift;
+        my $arg         = shift;
+        my ($start, $end);
+        if ($arg)
+                {
+                ($start, $end) = translate_range($arg, shift);
+                }
+        $start //= 0; $end //= -1;
+        my @list = ();
+        my $elt = $self->get_column($start);
+        my $last_elt = $self->get_column($end);
+        while ($elt && ! $elt->after($last_elt))
+                {
+                push @list, $elt;
+                $elt = $elt->next;
+                }
+        return @list;
         }
 
 sub     get_cell
         {
         my $self        = shift;
-        my ($r, $c) = ODF::lpOD::Table::translate_coordinates(@_);
+        my ($r, $c) = translate_coordinates(@_);
         my $row = $self->get_row($r)    or return undef;
         return $row->get_cell($c);
         }
 
+sub     get_cells
+        {
+        my $self        = shift;
+        my $arg         = shift;
+        
+        if (defined $arg)
+                {
+                $arg =~ s/ //g;
+                $arg = $arg ? uc($arg) : undef;
+                }
+
+        my ($r1, $r2, $c1, $c2);
+        given ($arg)
+                {
+                when (undef)
+                        {
+                        $r1 = 0; $r2 = -1; $c1 = 0; $c2 = -1;
+                        }
+                when (/[A-Z]/)
+                        {
+                        my ($a1, $a2) = split(':', $arg);
+                        ($r1, $c1) = translate_coordinates($a1);
+                        ($r2, $c2) = translate_coordinates($a2);
+                        }
+                when (/^[0-9]*$/)
+                        {
+                        $r1 = $arg; ($c1, $r2, $c2) = @_;
+                        }
+                default
+                        {
+                        alert "Wrong range definition syntax";
+                        return FALSE;
+                        }
+                }
+
+        my @t = ();
+        for (my $i = $r1, my $r = 0 ; $i <= $r2 ; $i++, $r++)
+                {
+                my $row = $self->get_row($i);
+                foreach (my $j = $c1, my $c = 0 ; $j <= $c2 ; $j++, $c++)
+                        {
+                        $t[$r][$c] = $row->get_cell($j);
+                        }
+                }
+
+        return @t;
+        }
+
+sub     get_cell_list
+        {
+        my $self        = shift;
+        return $self->get_cells(@_);
+        }
+
 sub     add_row
         {
         my $self        = shift;
@@ -290,12 +485,13 @@ sub     get_height
         my $self        = shift;
         my $height      = 0;
         my $row         = $self->first_child($ROW_FILTER);
+        my $max_h       = $self->att('#lpod:h');
         while ($row)
                 {
                 $height += $row->get_repeated;
-                $row = $row->next_sibling($ROW_FILTER);
+                $row = $row->next;
                 }
-        return $height;
+        return (defined $max_h and $max_h < $height) ? $max_h : $height;
         }
 
 sub     get_column_count
@@ -303,12 +499,13 @@ sub     get_column_count
         my $self        = shift;
         my $count       = 0;
         my $col         = $self->first_child($COLUMN_FILTER);
+        my $max_w       = $self->att('#lpod:w');
         while ($col)
                 {
                 $count += $col->get_repeated;
-                $col = $col->next_sibling($COLUMN_FILTER);
+                $col = $col->next;
                 }
-        return $count;        
+        return (defined $max_w and $max_w < $count) ? $max_w : $count;        
         }
 
 sub     get_size
@@ -317,13 +514,17 @@ sub     get_size
         my $height      = 0;
         my $width       = 0;
         my $row         = $self->first_child($ROW_FILTER);
+        my $max_h       = $self->att('#lpod:h');
+        my $max_w       = $self->att('#lpod:w');
         while ($row)
                 {
                 $height += $row->get_repeated;
                 my $row_width = $row->get_width;
                 $width = $row_width if $row_width > $width;
-                $row = $row->next_sibling($ROW_FILTER);
+                $row = $row->next;
                 }
+        
+        $height = $max_h if defined $max_h and $max_h < $height;
         return ($height, $width);
         }
 
@@ -348,7 +549,7 @@ sub     contains
 package ODF::lpOD::Column;
 use base 'ODF::lpOD::Element';
 our $VERSION    = 0.1;
-use constant PACKAGE_DATE => '2010-06-10T12:38:06';
+use constant PACKAGE_DATE => '2010-06-22T15:02:46';
 use ODF::lpOD::Common;
 #-----------------------------------------------------------------------------
 
@@ -373,6 +574,16 @@ sub     create
 
 #-----------------------------------------------------------------------------
 
+sub     repeat
+        {
+        my $self        = shift;
+        my $reps        = shift || $self->get_repeated;
+        $self->set_repeated(undef);
+        return $self->SUPER::repeat($reps);
+        }
+
+#-----------------------------------------------------------------------------
+
 sub     get_repeated
         {
         my $self        = shift;
@@ -382,7 +593,9 @@ sub     get_repeated
 sub     set_repeated
         {
         my $self        = shift;
-        return $self->set_attribute('table:number-columns-repeated', shift);
+        my $rep         = shift;
+        $rep = undef unless $rep && $rep > 1;
+        return $self->set_attribute('table:number-columns-repeated', $rep);
         }
 
 #=============================================================================
@@ -391,7 +604,7 @@ sub     set_repeated
 package ODF::lpOD::Row;
 use base 'ODF::lpOD::Element';
 our $VERSION    = 0.1;
-use constant PACKAGE_DATE => '2010-06-14T21:37:23';
+use constant PACKAGE_DATE => '2010-06-22T19:24:38';
 use ODF::lpOD::Common;
 #-----------------------------------------------------------------------------
 
@@ -418,6 +631,25 @@ sub     create
         return $row;
         }
 
+
+#-----------------------------------------------------------------------------
+
+sub     repeat
+        {
+        my $self        = shift;
+        my $reps        = shift || $self->get_repeated;
+        $self->set_repeated(undef);
+        return $self->SUPER::repeat($reps);
+        }
+
+sub     clean
+        {
+        my $self        = shift;
+        my $cell        = $self->last_child($CELL_FILTER)
+                or return undef;
+        $cell->set_repeated(undef);
+        }
+
 #-----------------------------------------------------------------------------
 
 sub     get_cell
@@ -425,6 +657,8 @@ sub     get_cell
         my $self        = shift;
         my $position    = shift || 0;
         my $width       = $self->get_width;
+        my $max_w       = $self->parent->get_attribute('#lpod:w');
+
         if ($position < 0)
                 {
                 $position += $width;
@@ -438,12 +672,36 @@ sub     get_cell
         my $cell = $self->first_child($CELL_FILTER)
                 or return undef;
         my $p = 0;
+        my $next_elt;
         do      {
-                my $next_elt = $cell->next_sibling($CELL_FILTER);
-                $p += $cell->repeat(); $p++;
-                $cell = $next_elt;
+                $next_elt = $cell->next;
+                $p = ODF::lpOD::Table::split_rep($cell, $p, $max_w);
+                $p++; $cell = $next_elt;
                 } until $p >= $position;
-        return $self->child($position, $CELL_FILTER);
+        $cell = $self->child($position, $CELL_FILTER);
+        ODF::lpOD::Table::split_rep($cell, $p, $max_w);
+        return $cell;
+        }
+
+sub     get_cell_list
+        {
+        my $self        = shift;
+        my $arg         = shift;
+        unless ($arg)
+                {
+                return $self->children($CELL_FILTER);
+                }
+        my ($start, $end) = ODF::lpOD::Table::translate_range($arg, shift);
+        $start //= 0; $end //= -1;
+        my @list = ();
+        my $elt = $self->get_cell($start);
+        my $last_elt = $self->get_cell($end);
+        while ($elt && ! $elt->after($last_elt))
+                {
+                push @list, $elt;
+                $elt = $elt->next;
+                }
+        return @list;
         }
 
 sub     get_width
@@ -451,12 +709,13 @@ sub     get_width
         my $self        = shift;
         my $width       = 0;
         my $cell        = $self->first_child($CELL_FILTER);
+        my $max_w       = $self->parent->att('#lpod:w');
         while ($cell)
                 {
                 $width += $cell->get_repeated;
-                $cell = $cell->next_sibling($CELL_FILTER);
+                $cell = $cell->next;
                 }
-        return $width;
+        return (defined $max_w and $max_w < $width) ? $max_w : $width;
         }
 
 sub     add_cell
@@ -525,7 +784,9 @@ sub     get_repeated
 sub     set_repeated
         {
         my $self        = shift;
-        return $self->set_attribute('table:number-rows-repeated', shift);
+        my $rep         = shift;
+        $rep = undef unless $rep && $rep > 1;
+        return $self->set_attribute('table:number-rows-repeated', $rep);
         }
 
 #=============================================================================
@@ -534,7 +795,7 @@ sub     set_repeated
 package ODF::lpOD::Cell;
 use base 'ODF::lpOD::Field';
 our $VERSION    = 0.1;
-use constant PACKAGE_DATE => '2010-06-19T19:59:45';
+use constant PACKAGE_DATE => '2010-06-22T19:24:10';
 use ODF::lpOD::Common;
 #-----------------------------------------------------------------------------
 our     %ATTRIBUTE;
@@ -548,10 +809,41 @@ sub     create
 
 #-----------------------------------------------------------------------------
 
+sub     is_covered
+        {
+        my $self        = shift;
+        my $tag         = $self->get_tag;
+        return $tag =~ /covered/ ? TRUE : FALSE;
+        }
+
+sub     next
+        {
+        my $self        = shift;
+        return $self->next_sibling($ODF::lpOD::Row::CELL_FILTER);
+        }
+
+sub     previous
+        {
+        my $self        = shift;
+        return $self->previous_sibling($ODF::lpOD::Row::CELL_FILTER);
+        }
+
+sub     repeat
+        {
+        my $self        = shift;
+        my $reps        = shift || $self->get_repeated;
+        $self->set_repeated(undef);
+        return $self->SUPER::repeat($reps);
+        }
+
+#-----------------------------------------------------------------------------
+
 sub     set_repeated
         {
         my $self        = shift;
-        return $self->set_attribute('table:number-columns-repeated', shift);
+        my $rep         = shift;
+        $rep = undef unless $rep && $rep > 1;
+        return $self->set_attribute('table:number-columns-repeated', $rep);
         }
 
 sub     get_repeated
